use bevy::{
    prelude::*,
    render::{
        primitives::Aabb,
        view::{NoFrustumCulling, VisibilitySystems},
    },
    sprite::MaterialMesh2dBundle,
    window::PrimaryWindow,
};
use bevy_rapier2d::parry::simba::scalar::SupersetOf;

#[derive(Component)]
pub struct ParallaxTile(usize);

#[derive(Component, Clone)]
pub struct Parallax {
    pub offset: f32,
    pub factor: f32,
    pub y_factor: f32,
    pub size: Vec2,
    pub depth: f32,
    pub tiles: usize,
    pub tex: String,
}

impl Default for Parallax {
    fn default() -> Self {
        Self {
            offset: 0.,
            factor: 10.,
            y_factor: 10.,
            size: Vec2::new(2000., 1000.),
            depth: 100.,
            tiles: 4,
            tex: "".to_owned(),
        }
    }
}

pub fn parallax_plugin(app: &mut App) {
    app.add_systems(Update, (update_parallax,));
}

pub fn parallax(
    commands: &mut Commands,
    materials: &mut ResMut<Assets<ColorMaterial>>,
    meshes: &mut ResMut<Assets<Mesh>>,
    assets: &Res<AssetServer>,
    parallax_info: Parallax,
) {
    for i in 0..parallax_info.tiles {
        commands.spawn((
            ParallaxTile(i),
            parallax_info.clone(),
            MaterialMesh2dBundle {
                mesh: meshes.add(Rectangle::from_size(parallax_info.size)).into(),
                material: materials.add(assets.load(parallax_info.tex.clone())),
                transform: Transform::from_xyz(0.0, 0.0, -parallax_info.depth),
                visibility: Visibility::Visible,
                ..Default::default()
            },
            NoFrustumCulling,
        ));
    }
}

fn update_parallax(
    mut commands: Commands,
    mut query: Query<(Entity, &Parallax, &ParallaxTile, &mut Transform)>,
    camera_query: Query<(&Camera2d, &Transform), (Without<Parallax>)>,
    window_query: Query<&Window, With<PrimaryWindow>>,
) {
    let (_, cam_transform) = camera_query.single();
    let cam_transform = cam_transform.translation;
    let window = window_query.single();
    for (entity, parallax_info, ParallaxTile(tile), mut transform) in query.iter_mut() {
        let position_on_parallax = (cam_transform.x / parallax_info.factor) % parallax_info.size.x;
        let first_parallax_on_cam = cam_transform.x - position_on_parallax;
        let position = Vec2::new(
            first_parallax_on_cam + *tile as f32 * parallax_info.size.x,
            cam_transform.y,
        );
        transform.translation =
            position.extend(-parallax_info.depth) + Vec3::new(0., parallax_info.offset, 0.);
    }
}
